//-----------------------------------------------------------------------------
/*!
   \file          cb.c
   \brief         circular buffer functions


   \implementation
    project       Wens 700 display
    version       0.00
    generated     Matthias Koelling 23.10.2021
    compiler      AVR GCC
    target        ATmega164PA
   \endimplementation
*/
//-----------------------------------------------------------------------------

/* -- Configuration Parameters --------------------------------------------- */

/* -- Includes ------------------------------------------------------------- */

#include <stddef.h>
#include <stdbool.h>
#include <string.h>
#include <stdlib.h>
#include "cb.h"

/* -- Makros --------------------------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */

/* -- Types ---------------------------------------------------------------- */

/* -- Module Global Variables ---------------------------------------------- */

/* -- Extern Global Variables ---------------------------------------------- */

/* -- Makros --------------------------------------------------------------- */

/* -- Implementation Functions --------------------------------------------- */

//-----------------------------------------------------------------------------
/*!
   \brief      cb_init

               timer initialization

   \param[in]  t_capacity -> number of elements
               t_size -> size of 1 element
   \param[out] opt_cb -> buffer discriptor
   \return     none
*/
//-----------------------------------------------------------------------------
void cb_init (T_circular_buffer* opt_cb, size_t opt_capacity, size_t opt_size)
{
   opt_cb->pv_buffer = malloc (opt_capacity * opt_size);
   opt_cb->pv_buffer_end = (char*)opt_cb->pv_buffer + opt_capacity * opt_size;
   opt_cb->t_capacity = opt_capacity;
   opt_cb->t_count = 0;
   opt_cb->t_size = opt_size;
   opt_cb->pv_head = opt_cb->pv_buffer;
   opt_cb->pv_tail = opt_cb->pv_buffer;
}

//-----------------------------------------------------------------------------
/*!
   \brief      cb_free


   \param[in]  opt_cb -> buffer descriptor

   \return     none
*/
//-----------------------------------------------------------------------------
void cb_free (T_circular_buffer *opt_cb)
{
    free (opt_cb->pv_buffer);
    // clear out other fields too, just to be safe
}

//-----------------------------------------------------------------------------
/*!
   \brief      cb_push_back


   \param[in]  opt_cb -> buffer descriptor
               opv_item -> element to insert

   \return     void
*/
//-----------------------------------------------------------------------------
void cb_push_back (T_circular_buffer* opt_cb, const void* opv_item)
{
   if (opt_cb->t_count != opt_cb->t_capacity)
   {
      memcpy (opt_cb->pv_head, opv_item, opt_cb->t_size);
      opt_cb->pv_head = (char*)opt_cb->pv_head + opt_cb->t_size;

      if (opt_cb->pv_head == opt_cb->pv_buffer_end)
      {
         opt_cb->pv_head = opt_cb->pv_buffer;
      }
      opt_cb->t_count++;
   }
}

//-----------------------------------------------------------------------------
/*!
   \brief      cb_pop_front

   \param[in]  opt_cb -> buffer descriptor
   \param[out] opv_item -> element to read

   \return     void
*/
//-----------------------------------------------------------------------------
bool cb_pop_front (T_circular_buffer* opt_cb, void* opv_item)
{
   if (opt_cb->t_count != 0)
   {
      memcpy (opv_item, opt_cb->pv_tail, opt_cb->t_size);
      opt_cb->pv_tail = (char*)opt_cb->pv_tail + opt_cb->t_size;
      if (opt_cb->pv_tail == opt_cb->pv_buffer_end)
      {
         opt_cb->pv_tail = opt_cb->pv_buffer;
      }
      opt_cb->t_count--;
      return true;
   }
   return false;
}

// EOF
