//-----------------------------------------------------------------------------
/*!
   \file           ioport.h
   \brief          header for ioport.c

   \implementation
    project       tube radio power supply
    version       0.00
    generated     Matthias Koelling 20.03.2013
    compiler      AVR GCC
    target        ATTiny13
   \endimplementation
*/
//-----------------------------------------------------------------------------

#ifndef IOPORT_H
#define IOPORT_H

/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>

/* -- Configuration Parameters --------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */

#define S1D13305_OUT(ou8_data) {\
                                   DDRC = 0xFF;\
                                   PORTC = ou8_data;\
                               }
#define S1D13305_IN(ou8_data) {\
                                  DDRC = 0;\
                                  ou8_data = PINC;\
                              }
#define UC1698_OUT(ou8_data) {\
                                 DDRA = 0xFF;\
                                 PORTA = ou8_data;\
                             }

#define S1D_IN(ou8_data) {\
                             DDRA = 0;\
                             ou8_data = PINA;\
                         }
#define S1D13305_WR      SBIT(PIND,3)
#define S1D13305_RD      SBIT(PIND,4)
#define S1D13305_RES     SBIT(PIND,5)
#define S1D13305_A0      SBIT(PIND,6)

#define  UC1698_RST      SBIT(PINB,0)
#define  UC1698_WR       SBIT(PINB,1)
#define  UC1698_RD       SBIT(PINB,2)
#define  UC1698_A0       SBIT(PINB,3)

#define ON                      (1u)
#define OFF                     (0u)


/* -- Types ---------------------------------------------------------------- */

typedef struct
{
   volatile uint8_t* pu8_DIR_Reg;          // data direction register
   volatile uint8_t* pu8_IO_Reg;           // in/out register for read/write
   uint8_t u8_Pin;                         // which pin does the signal use
   uint8_t u8_SignalType;                  // input or output
   uint8_t u8_Active_Level;
}
T_ioport_config;

/* -- Extern Global Variables ---------------------------------------------- */

/* -- Global Function Prototypes ------------------------------------------- */

extern void ioport_init (void);
extern uint8_t ioport_pin_get (const uint8_t ou8_channel);
extern void ioport_pin_set (const uint8_t ou8_channel, const uint8_t ou8_value);
extern void ioport_debounce (void);

#endif // IOPORT_H

// EOF
