//-----------------------------------------------------------------------------
/*!
   \file          state.c
   \brief         state machine


   \implementation
    project       tube radio power supply
    version       0.00
    generated     Matthias Koelling 11.12.2020
    compiler      AVR GCC
    target        ATTiny13
   \endimplementation
*/
//-----------------------------------------------------------------------------

/* -- Configuration Parameters --------------------------------------------- */

/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>
#include "ioport.h"
#include "timer.h"
#include "state.h"

/* -- Defines -------------------------------------------------------------- */
#define STATE_OFF             (0u)
#define STATE_START           (1u)
#define STATE_ENABLE_PULSE    (2u)
#define STATE_ENABLED         (3u)
#define STATE_SWITCH_POWERED  (4u)
#define STATE_VIN_POWERED     (5u)

#define PULSE_HIGH (100u)
#define INDICATOR_REFRESH (1000u)
#define PULSE_TIME (PULSE_HIGH / TIMER_CYCLE_TIME)
#define INDICATOR_TIME (INDICATOR_REFRESH / TIMER_CYCLE_TIME)

/* -- Types ---------------------------------------------------------------- */

/* -- Module Global Variables ---------------------------------------------- */

/* -- Extern Global Variables ---------------------------------------------- */

/* -- Implementation ------------------------------------------------------- */

//-----------------------------------------------------------------------------
/*!
   \brief      state_task

               main state machine

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------
void state_task (void)
{
   static uint8_t hu8_state = STATE_OFF;
   static uint8_t hu8_time_counter = 0;

   if (ioport_pin_get (IOPORT_SWITCH) == 0)
   {
      ioport_pin_set (IOPORT_ENABLE, OFF);
      ioport_pin_set (IOPORT_MOSFET, OFF);
      hu8_state = STATE_OFF;
   }
   switch (hu8_state)
   {
   case STATE_OFF:
      if (ioport_pin_get (IOPORT_SWITCH) == 1)
      {
         hu8_state = STATE_START;
      }
      break;
   case STATE_START:
      ioport_pin_set (IOPORT_ENABLE, ON);
      hu8_time_counter = PULSE_TIME;
      hu8_state = STATE_ENABLE_PULSE;
      break;
   case STATE_ENABLE_PULSE:
      hu8_time_counter--;
      if (hu8_time_counter == 0)
      {
         ioport_pin_set (IOPORT_ENABLE, OFF);
         hu8_state = STATE_ENABLED;
      }
      break;
   case STATE_ENABLED:
      ioport_pin_set (IOPORT_MOSFET, ON);
      hu8_state = STATE_SWITCH_POWERED;
      hu8_time_counter = INDICATOR_TIME;
      break;
   case STATE_SWITCH_POWERED:
      hu8_time_counter--;
      if (hu8_time_counter == 0)
      {
         hu8_state = STATE_START;
      }
      break;
   }
}

//ISR (PCINT0_vect)
//{
//   if (VIN == 0) // falling edge
//   {
//      if (status == VIN_POWERED)
//      {
//         ENABLE_ON;
//         TCNT0 = (256 - 65); // 65  * 1707us ==> 111ms
//         TIMSK0 |= (1 << OCIE0A); // enable timer 0 overflow interrupt
//         status = SWITCH_POWERED;
//      }
//   }
//   else // rising edge
//   {
//      if (status == OFF)
//      {
//         // in this case we must suppress the enable pulse
//         // because this would switch of VBUS
//         status = VIN_POWERED;
//      }
//   }
//}

// EOF
