//-----------------------------------------------------------------------------
/*!
   \file          uart.c
   \brief         uart functions


   \implementation
    project       Wens 700 display
    version       0.00
    generated     Matthias Koelling 23.10.2021
    compiler      AVR GCC
    target        ATmega164PA
   \endimplementation
*/
//-----------------------------------------------------------------------------

/* -- Configuration Parameters --------------------------------------------- */

/* -- Includes ------------------------------------------------------------- */

#include <stdbool.h>
#include <avr/io.h>
#include "uart.h"

/* -- Makros --------------------------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */

/* -- Types ---------------------------------------------------------------- */


/* -- Module Global Variables ---------------------------------------------- */

/* -- Extern Global Variables ---------------------------------------------- */

/* -- Makros --------------------------------------------------------------- */

/* -- Implementation Functions --------------------------------------------- */

//-----------------------------------------------------------------------------
/*!
   \brief      uart_init

               uart initialization

   \param[in]  ou16_bit_rate -> bitrate
   \return     void
*/
//-----------------------------------------------------------------------------
void uart_init (const uint32_t ou32_bit_rate)
{
   uint16_t u16_bitrate = (F_CPU / (ou32_bit_rate  * 16)) - 1;

   UBRR0H = u16_bitrate >> 8;
   UBRR0L = u16_bitrate & 0xFF;
   UCSR0C = (1 << UCSZ01) | (1 << UCSZ00); // Asynchron 8N1
   UCSR0B |= (1<<RXEN0)|(1<<TXEN0);  // UART RX, TX einschalten
}

//-----------------------------------------------------------------------------
/*!
   \brief      uart_send


   \param[in]  ou8_data -> data to send
   \return     timer flag
*/
//-----------------------------------------------------------------------------
bool uart_send (const uint8_t ou8_data)
{
   if(UCSR0A & (1<<UDRE0))         /* Senden, wenn UDR frei ist                    */
   {
      UDR0 = ou8_data;              /* schreibt das Zeichen x auf die Schnittstelle */
      return true;
   }
   return false;
}

// EOF
